package pkg

import (
	"github.com/julienschmidt/httprouter"
	"net/http"
)

const (
	YearPeriod       = "year"
	QuarterPeriod    = "quarter"
	MonthPeriod      = "month"
	YesteryearPeriod = "yesteryear"
)

type DashboardPage struct {
	Sales     string
	Income    string
	Expenses  string
	VAT       string
	IRPF      string
	NetIncome string
	Filters   *dashboardFilterForm
}

func ServeDashboard(w http.ResponseWriter, r *http.Request, _ httprouter.Params) {
	company := mustGetCompany(r)
	locale := getLocale(r)
	filters := newDashboardFilterForm(locale, company)
	if err := filters.Parse(r); err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}
	periodStart := "30 DAYS"
	periodEnd := "0 DAYS"
	switch filters.Period.Selected {
	case YearPeriod:
		periodStart = "1 YEAR"
	case QuarterPeriod:
		periodStart = "3 MONTHS"
	case YesteryearPeriod:
		periodStart = "2 YEARS"
		periodEnd = "1 YEAR"
	case "":
		filters.Period.Selected = MonthPeriod
	}
	conn := getConn(r)
	rows := conn.MustQuery(r.Context(), `
		select to_price(0, decimal_digits) as sales
		     , to_price(coalesce(invoice.total, 0), decimal_digits) as income
		     , to_price(coalesce(expense.total, 0), decimal_digits) as expenses
		     , to_price(coalesce(tax.vat, 0), decimal_digits) as vat
		     , to_price(coalesce(tax.irpf, 0), decimal_digits) as irpf
		     , to_price(coalesce(invoice.total, 0) - coalesce(expense.total, 0) - coalesce(tax.vat, 0) + coalesce(tax.irpf, 0), decimal_digits) as net_income
		from company
		    left join (
		        select company_id, sum(total)::integer as total
		        from invoice
		    	join invoice_amount using (invoice_id)
		        where invoice_date between CURRENT_DATE - $2::interval and CURRENT_DATE - $3::interval
		        group by company_id
		    ) as invoice using (company_id)
		    left join (
		        select company_id, sum(amount)::integer as total
		        from expense
		        where invoice_date between CURRENT_DATE - $2::interval and CURRENT_DATE - $3::interval
		        group by company_id
		    ) as expense using (company_id)
		    left join (
		        select invoice.company_id
		             , sum(case when tax_class.name = 'IVA' then invoice_tax_amount.amount else 0 end)::integer as vat
		             , sum(case when tax_class.name = 'IRPF' then invoice_tax_amount.amount else 0 end)::integer as irpf
		        from invoice
		        join invoice_tax_amount using (invoice_id)
		        join tax using (tax_id)
		        join tax_class using (tax_class_id)
		        where invoice_date between CURRENT_DATE - $2::interval and CURRENT_DATE - $3::interval
		        group by invoice.company_id
		    ) as tax using (company_id)
		join currency using (currency_code)
		where company_id = $1
	`, company.Id, periodStart, periodEnd)
	defer rows.Close()

	dashboard := &DashboardPage{
		Filters: filters,
	}
	for rows.Next() {
		if err := rows.Scan(&dashboard.Sales, &dashboard.Income, &dashboard.Expenses, &dashboard.VAT, &dashboard.IRPF, &dashboard.NetIncome); err != nil {
			panic(err)
		}
	}
	if rows.Err() != nil {
		panic(rows.Err())
	}
	mustRenderMainTemplate(w, r, "dashboard.gohtml", dashboard)
}

type dashboardFilterForm struct {
	locale  *Locale
	company *Company
	Period  *RadioField
}

func newDashboardFilterForm(locale *Locale, company *Company) *dashboardFilterForm {
	return &dashboardFilterForm{
		locale:  locale,
		company: company,
		Period: &RadioField{
			Name:     "period",
			Selected: MonthPeriod,
			Label:    pgettext("input", "Period", locale),
			Options: []*RadioOption{
				{
					Label: pgettext("period option", "Year", locale),
					Value: YearPeriod,
				},
				{
					Label: pgettext("period option", "Quarter", locale),
					Value: QuarterPeriod,
				},
				{
					Label: pgettext("period option", "Month", locale),
					Value: MonthPeriod,
				},
				{
					Label: pgettext("period option", "Previous Year", locale),
					Value: YesteryearPeriod,
				},
			},
		},
	}
}

func (form *dashboardFilterForm) Parse(r *http.Request) error {
	if err := r.ParseForm(); err != nil {
		return err
	}
	form.Period.FillValue(r)
	return nil
}
